# 2arm Controller

Ros MoveIt module for 2 UR5e arms sharing a workspace.

Will be developed for setup being built with rotating print bed.

#### TO DO

- [x] Create xacro with two ur5e's in it
- [x] Create a moveit package using the xacro
- [x] Create table as collision object to plan around
- [x] Link python script which can control arms
- [x] Create python file which can be imported for controlling robot
- [x] Create interface for following contours from gcode2contour
- [ ] Get (ideally) IKFast or tracIK working for better IK results
- [x] See if collision free paths can be calculated with moving objects
- [ ] Looks like the following will need openrave, so need to set that up and link the environments
- [ ] Create interface to collision check given a robot arms in a given orientation
- [ ] Create a planner class which holds plans and can be queried for orientation given time
- [ ] Create a planner for moving objects, local search approach


## Building/Installing

#### Requirements

Only tested for the following setup.
- [Ubuntu 20.04](https://ubuntu.com/download/desktop) (ROS will not work with 20.10)
- [ROS Noetic](http://wiki.ros.org/noetic/Installation)
- [MoveIt](https://moveit.ros.org/install/)
- [Miniconda](https://docs.conda.io/projects/conda/en/latest/user-guide/install/)

**Note**: Going to try and use openrave via docker container instead of installing since installation isn't easy/repeatable.
- Install Docker
- `docker pull hamzamerzic/openrave`

#### Install

```bash
# clone with submodules
git clone --recurse-submodules https://code.research.uts.edu.au/bigprint/2arm-controller.git

# create conda environment containing python packages needed for ros
cd 2arm-controller
conda env create -f rosenv.yml
conda activate ros
# conda env cannot install gcode2contour
pip install git+https://code.research.uts.edu.au/bigprint/gcode2contour.git

# make and source
cd catkin_ws
catkin_make
source devel/setup.zsh  # or .bash, do this in every new shell
```

## Running

To create a simulation environment, run the following:

```bash
# Launch a simulation instance and visualise in RViz
roslaunch 2arm_moveit demo.launch
```

To control the arms programatically from python:

```python
from two_arm_utils import TwoArmController

# MoveIt must be running
tac = TwoArmController()

# Move arm 1 to vertical position
tac.go_to_joint([0,0,0,0,0,0], 1)

# Move arm 2's end effector pose
pose1 = tac.arm2.get_current_pose()
pose1.pose.position.x += 0.2 # 20cm
tac.go_to_pose(pose1, 2)
```
